# Copyright (c) 2010, 2011, Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Tests for the oops config module."""

__metaclass__ = type

from functools import partial

import testtools
from testtools.matchers import Is

from oops.config import Config
from oops.createhooks import default_hooks


class TestConfig(testtools.TestCase):

    def test_init(self):
        config = Config()
        self.assertEqual(default_hooks, config.on_create)
        self.assertEqual([], config.publishers)
        self.assertEqual({}, config.template)

    def test_on_create_called(self):
        calls = []
        def capture(id, report, context):
            calls.append((id, report, context))
        config = Config()
        config.on_create = []
        config.on_create.append(partial(capture, '1'))
        config.on_create.append(partial(capture, '2'))
        report = config.create(dict(foo='2'))
        self.assertThat(report, Is(calls[0][1]))
        self.assertThat(report, Is(calls[1][1]))
        self.assertEqual(
                [('1', {}, {'foo':'2'}), ('2', {}, {'foo':'2'})], calls)

    def test_on_create_no_context(self):
        calls = []
        def capture(report, context):
            calls.append((report, context))
        config = Config()
        config.on_create = []
        config.on_create.append(capture)
        report = config.create()
        self.assertEqual([({}, {})], calls)

    def test_create_template(self):
        config = Config()
        config.on_create = []
        config.template['base'] = True
        config.template['list'] = []
        report = config.create()
        self.assertEqual({'base': True, 'list': []}, report)
        self.assertIsNot(report['list'], config.template['list'])

    def test_publish_calls_publishers(self):
        calls = []
        def pub_1(report):
            return '1'
        def pub_2(report):
            self.assertEqual('1', report['id'])
            return '2'
        config = Config()
        config.publishers.append(pub_1)
        config.publishers.append(pub_2)
        report = {}
        self.assertEqual(['1', '2'], config.publish(report))
        self.assertEqual({'id': '2'}, report)

    def test_publish_filters_first(self):
        calls = []
        def filter_ok(report):
            calls.append('ok')
        def filter_block(report):
            calls.append('block')
            return True
        def pub(report):
            calls.append('pub')
            return '1'
        config = Config()
        config.filters.append(filter_ok)
        config.filters.append(filter_block)
        config.filters.append(filter_ok)
        config.publishers.append(pub)
        report = {}
        self.assertEqual(None, config.publish(report))
        self.assertEqual(['ok', 'block'], calls)
